package service

import (
	"github.com/CodFrm/cxmooc-tools/server/application/dto"
	"github.com/CodFrm/cxmooc-tools/server/application/event/publish"
	"github.com/CodFrm/cxmooc-tools/server/domain/entity"
	"github.com/CodFrm/cxmooc-tools/server/domain/repository"
	"time"
)

type Topic struct {
	topicRepo    repository.TopicRepository
	integralRepo repository.IntegralRepository
	userRepo     repository.UserRepository
}

func NewTopicDomainService(topicRepo repository.TopicRepository) *Topic {
	return &Topic{
		topicRepo: topicRepo,
	}
}

func (t *Topic) SearchTopicList(topic []string) ([]dto.TopicSet, error) {
	ret := make([]dto.TopicSet, 0)
	for k, v := range topic {
		entity := new(entity.TopicEntity)
		entity.SetTopic(v)
		if entities, err := t.topicRepo.SearchTopic(entity); err != nil {
			return nil, err
		} else {
			ret = append(ret, dto.TopicSet{
				Index:  k,
				Result: dto.ToSearchResults(entities),
				Topic:  v,
			})
		}
	}
	return ret, nil
}

func (t *Topic) SubmitTopic(topic []*dto.SubmitTopic, ip, platform, token string, isImport bool) ([]dto.TopicHash, *dto.InternalAddMsg, error) {
	ret := make([]dto.TopicHash, 0)
	addNum := &dto.InternalAddMsg{}
	for _, v := range topic {
		et := dto.ToTopicEntity(v, ip, platform, token)
		if len(v.Correct) <= 0 {
			continue
		}
		if ok, err := t.topicRepo.Exist(et); err != nil {
			return nil, nil, err
		} else if ok && (v.Type == 2 || v.Type == 4) {
			//合并
			var err error
			et, err = t.topicRepo.FindByHash(et.GetHash())
			if err != nil {
				return nil, nil, err
			}
			et = t.mergeAnswer(et, v)
			et.UpdateTime = time.Now().Unix()
			if err := t.topicRepo.Save(et); err != nil {
				return nil, nil, err
			}
		} else if !ok {
			et.CreateTime = time.Now().Unix()
			if err := t.topicRepo.Save(et); err != nil {
				return nil, nil, err
			}
			addNum.AddTokenNum += 10
		}
		ret = append(ret, dto.TopicHash{Hash: et.GetHash()})
	}
	if addNum.AddTokenNum > 0 {
		if isImport {
			publish.ImportTopic(token, addNum.AddTokenNum/10)
		} else {
			publish.SubmitTopic(token, addNum.AddTokenNum/10)
		}
	}
	return ret, addNum, nil
}

func (t *Topic) mergeAnswer(et *entity.TopicEntity, d *dto.SubmitTopic) *entity.TopicEntity {
	if len(et.Correct) >= len(d.Correct) {
		return et
	}
	source := dto.MapToAnswerValue(d.Correct, d.Type)
	for _, new := range source {
		flag := true
		for _, old := range et.Correct {
			if old.Option == new.Option {
				flag = false
				break
			}
		}
		if flag {
			et.Correct = append(et.Correct, new)
		}
	}
	return et
}
